<?php

namespace Drupal\xray_audit\Plugin\tasks\QueryGroup;

/**
 * Plugin implementation of queries_data_node.
 *
 * @XrayAuditTaskPlugin (
 *   id = "queries_data_node",
 *   label = @Translation("Data about Nodes"),
 *   description = @Translation("Queries execute on database to get reports about nodes."),
 *   group = "queries_data",
 *   sort = 2,
 *   operations = {
 *      "node_by_types" = {
 *          "label" = "Number of node types",
 *          "description" = "Node types, number of published and unpublished, and totals of nodes in the default language." *
 *       },
 *       "node_types_per_language_language" = {
 *          "label" = "Number of node types per language",
 *          "description" = "Number of node tupes per language"
 *       }
 *    },
 *   dependencies = {"node"}
 * )
 */
class XrayAuditQueryTaskNodePlugin extends XrayAuditQueryTaskPluginBase {

  /**
   * {@inheritdoc}
   */
  public function getDataResulOperation(string $operation = NULL) {
    switch ($operation) {
      case 'node_by_types':
        return $this->nodeByTypes();
      case 'node_types_per_language_language':
        return $this->nodeTypesPerLanguage();
    }
    return [];
  }

  /**
   * Get data for operation "node_by_types".
   *
   * @return array
   *   Render array.
   */
  protected function nodeByTypes() {

    $headerTable = [
      $this->t('label'),
      $this->t('id'),
      $this->t('langcode'),
      $this->t('total'),
      $this->t('publish'),
      $this->t('no-publish'),
    ];
    $resultTable = [];

    // Get label of content types.
    $label = [];
    $types = $this->entityTypeManager->getStorage("node_type")->loadMultiple();
    foreach ($types as $key => $type) {
      $label[$key] = $type->label();
    }

    $language_by_default = $this->languageManager->getDefaultLanguage()->getId();
    $query = $this->database->query("select type, langcode, count(1) as count, (select count(1) from node_field_data where status=1 and langcode = node.langcode and type = node.type ) as publish, (select count(1) from node_field_data where status != 1 and langcode = node.langcode and type = node.type) as no_publish from node_field_data as node where langcode='{$language_by_default}' group by type, langcode order by count DESC");
    $result = $query->fetchAll();
    $total = 0;
    $total_publish = 0;
    $total_no_publish = 0;
    foreach ($result as $row) {
      $resultTable[$row->type] = [
        'label' => $label[$row->type],
        'id' => $row->type,
        'langcode' => $row->langcode,
        'total' => $row->count ?? 0,
        'publish' => $row->publish ?? 0,
        'no_publish' => $row->no_publish ?? 0,
      ];
      $total += $row->count;
      $total_publish += $row->publish;
      $total_no_publish += $row->no_publish;
    }
    $resultTable['total']['label'] = 'TOTAL';
    $resultTable['total']['id'] = '---';
    $resultTable['total']['langcode'] = '---';
    $resultTable['total']['total'] = $total;
    $resultTable['total']['publish'] = $total_publish;
    $resultTable['total']['no-publish'] = $total_no_publish;

    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];

  }

  /**
   * Get data for operation "node_by_types_language".
   *
   * @return array
   *   Render array.
   */
  protected function nodeTypesPerLanguage () {

    $headerTable = [
      $this->t('label'),
      $this->t('id'),
      $this->t('langcode'),
      $this->t('total'),
    ];
    $resultTable = [];

    // Get label of content types.
    $label = [];
    $types = $this->entityTypeManager->getStorage("node_type")->loadMultiple();
    foreach ($types as $key => $type) {
      $label[$key] = $type->label();
    }

    $query = $this->database->query("select langcode, type, count(1) as count from node_field_data as node group by langcode, type");
    $result = $query->fetchAll();
    foreach ($result as $row) {
      $resultTable[] = [
        'label' => $label[$row->type],
        'id' => $row->type,
        'langcode' => $row->langcode,
        'total' => $row->count ?? 0,
      ];
    }

    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];

  }

  /**
   * {@inheritdoc}
   */
  public function buildDataPresentationForCli(array $data, string $operation = NULL) {
    // @todo Implement buildDataPresentationForCli() method.
    return '';
  }

}
