<?php

namespace Drupal\xray_audit\Plugin\tasks\QueryGroup;

/**
 * Plugin implementation of queries_data_node.
 *
 * @XrayAuditTaskPlugin (
 *   id = "queries_data_paragraphs",
 *   label = @Translation("Data about paragraphs"),
 *   description = @Translation("Queries execute on database to get reports about paragraphs."),
 *   group = "queries_data",
 *   sort = 4,
 *   operations = {
 *      "paragraphs_count" = {
 *          "label" = "Paragraphs types and count",
 *          "description" = "Paragraphs types and count."
 *       },
 *      "paragraphs_count_by_language" = {
 *          "label" = "Paragraphs count grouped by language",
 *          "description" = "Paragraphs count grouped by language."
 *       },
 *    },
 *   dependencies = {"paragraphs"}
 * )
 */
class XrayAuditQueryTaskParagraphsPlugin extends XrayAuditQueryTaskPluginBase {

  /**
   * {@inheritdoc}
   */
  public function getDataResulOperation(string $operation = NULL) {
    switch ($operation) {
      case 'paragraphs_count':
        return $this->paragraphsTypes();

      case 'paragraphs_count_by_language':
        return $this->paragraphsItemsCountByLanguage();
    }
    return [];
  }

  /**
   * Paragraphs types.
   */
  public function paragraphsTypes() {
    $headerTable = ['id', 'label', 'count'];
    $resultTable = [];

    $query = $this->database->query("SELECT type, count(1) as count FROM {paragraphs_item_field_data} GROUP by type");
    $result = $query->fetchAll();
    // Prepare results.
    $result_processed = [];
    foreach ($result as $row) {
      $result_processed[$row->type] = $row->count;
    }
    // dump($result_processed);
    $medias = $this->entityTypeManager->getStorage('paragraphs_type')->loadMultiple();
    foreach ($medias as $key => $media) {
      $resultTable[] = [
        $key,
        $media->label(),
        ($result_processed[$key] ?? 0),
      ];
    }
    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];
  }

  /**
   * Paragraphs items count by language.
   */
  public function paragraphsItemsCountByLanguage() {

    $headerTable = ['type', 'langcode', 'total'];
    $resultTable = [];

    $query = $this->database->query("SELECT type, langcode, COUNT(*) as total FROM {paragraphs_item_field_data} GROUP by type, langcode ASC");
    $result = $query->fetchAll();

    foreach ($result as $row) {
      $resultTable[] = [$row->type, $row->langcode, $row->total];
    }
    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];

  }

  /**
   * {@inheritdoc}
   */
  public function buildDataPresentationForCli(array $data, string $operation = NULL) {
    // @todo Implement buildDataPresentationForCli() method.
    return '';
  }

}
