<?php

namespace Drupal\xray_audit\Plugin\tasks\QueryGroup;

use Drupal\taxonomy\Entity\Vocabulary;

/**
 * Plugin implementation of queries_data_node.
 *
 * @XrayAuditTaskPlugin (
 *   id = "queries_data_vocabulary",
 *   label = @Translation("Data about vocabulary"),
 *   description = @Translation("Queries execute on database to get reports about vocabularies."),
 *   group = "queries_data",
 *   sort = 3,
 *   operations = {
 *      "vocabulary_description" = {
 *          "label" = "Vocabularies",
 *          "description" = "Vocabulary description."
 *       },
 *       "vocabulary_count" = {
 *          "label" = "Vocabulary count",
 *          "description" = "Vocabulary Count"
 *       },
 *      "vocabulary_count_per_language" = {
 *          "label" = "Vocabulary count per language",
 *          "description" = "Vocabulary count per language."
 *       }
 *    },
 *   dependencies = {"taxonomy"}
 *
 * )
 */
class XrayAuditQueryTaskVocabularyPlugin extends XrayAuditQueryTaskPluginBase {

  /**
   * {@inheritdoc}
   */
  public function getDataResulOperation(string $operation = NULL) {
    switch ($operation) {
      case 'vocabulary_description':
        return $this->vocabulariesDescription();

      case 'vocabulary_count':
        return $this->vocabulariesCount();

      case 'vocabulary_count_per_language':
        return $this->vocabulariesCountPerLanguage();
    }
    return [];
  }

  /**
   * Vocabularies description.
   */
  public function vocabulariesDescription() {
    $headerTable = ['id', 'label', 'description', 'langcode'];
    $resultTable = [];
    $vocabularies = Vocabulary::loadMultiple();
    foreach ($vocabularies as $key => $vocabulary) {
      $resultTable[] = [
        $key,
        $vocabulary->label(),
        $vocabulary->get('description'),
        $vocabulary->get('langcode'),
      ];
    }
    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];
  }

  /**
   * Vocabularies count.
   */
  public function vocabulariesCount() {
    $headerTable = ['vid', 'count'];
    $resultTable = [];
    $query = $this->database->query("SELECT vid, COUNT(1) as count FROM {taxonomy_term_field_data} GROUP by vid");
    $result = $query->fetchAll();
    foreach ($result as $row) {
      $resultTable[] = [$row->vid, $row->count];
    }
    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];
  }

  /**
   * Vocabularies count per language.
   */
  public function vocabulariesCountPerLanguage() {
    $headerTable = ['vid', 'langcode', 'count'];
    $resultTable = [];
    $query = $this->database->query("SELECT vid, langcode, COUNT(1) as count FROM {taxonomy_term_field_data} GROUP by vid, langcode;");
    $result = $query->fetchAll();
    foreach ($result as $row) {
      $resultTable[] = [$row->vid, $row->langcode, $row->count];
    }
    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildDataPresentationForCli(array $data, string $operation = NULL) {
    // @todo Implement buildDataPresentationForCli() method.
    return '';
  }

}
